"use strict";
import * as _ from "lodash";
import * as Common from "../Utils/Common";
import { readModel as permissionsReadModel, reducer as permissionsReducer } from "./Reducer";
export class Permissions {
    constructor(framework) {
        this.framework = framework;
        this.setAddressRole = async (acc, fromAddress, targetAddress, targetRole) => {
            let tx = await acc.setAddressRole(targetAddress, targetRole, {
                from: fromAddress,
                gas: 4000000
            });
            let fsa = Common.getFSAFromEventArgs(tx.logs[0].args);
            return {
                events: [fsa],
                tx: tx
            };
        };
        this.setGrant = async (acc, fromAddress, role, resource, action, attributes) => {
            let tx = await acc.setGrant(role, resource, action, attributes, {
                from: fromAddress,
                gas: 4000000
            });
            // second event is EsEvent...
            let fsa = Common.getFSAFromEventArgs(tx.logs[1].args);
            return {
                events: [fsa],
                tx: tx
            };
        };
        this.getGrant = async (acc, fromAddress, index) => {
            let grantVals = await acc.getGrant.call(index, {
                from: fromAddress
            });
            let grant = Common.grantItemFromValues(grantVals);
            return grant;
        };
        this.canRoleActionResource = async (acc, fromAddress, role, action, resource) => {
            let vals = await acc.canRoleActionResource.call(role, action, resource, {
                from: fromAddress
            });
            return vals[0];
        };
    }
    // NOT OPTIMIZED - WILL BE SLOW
    // NEED TO IMPLEMENT CACHIN...
    async getPermissionsReadModel(acc, fromAddress) {
        // console.log('getGrants...')
        let events = await this.framework.EventStore.readFSAs(acc, fromAddress, 0);
        // console.log(events)
        events = _.filter(events, (event) => {
            return event.type === "AC_GRANT_WRITTEN";
        });
        events = events.map(async (event) => {
            event.payload.grant = await this.getGrant(acc, fromAddress, event.payload.index);
            event.meta.extended = ["grant"];
            return event;
        });
        events = await Promise.all(events);
        let readModel = this.framework.ReadModel.readModelGenerator(permissionsReadModel, permissionsReducer, events);
        readModel.contractAddress = acc.address;
        readModel.readModelStoreKey = `${readModel.readModelType}:${readModel.contractAddress}`;
        return readModel;
    }
}
//# sourceMappingURL=Permissions.js.map