"use strict";
const bip39 = require("bip39");
const hdkey = require("ethereumjs-wallet/hdkey");
import * as Com from "./ECRecover/Common";
export class Toolbox {
    constructor(framework) {
        this.framework = framework;
        this.sign = async (address, message) => {
            return await Com.getMessageSignatureWithMeta(this.framework.web3, address, message);
        };
        this.recover = async (address, messageBufferHex, signature) => {
            const messageBuffer = Buffer.from(messageBufferHex.substring(2), "hex");
            return await Com.recoverAddressFromSig(messageBuffer, signature);
        };
        this.generateMnemonic = () => {
            return bip39.generateMnemonic();
        };
        this.getWalletFromMnemonic = (mnemonic) => {
            const hdwallet = hdkey.fromMasterSeed(bip39.mnemonicToSeed(mnemonic));
            // Get the first account using the standard hd path.
            const walletHDPath = "m/44'/60'/0'/0/";
            const wallet = hdwallet.derivePath(walletHDPath + "0").getWallet();
            return wallet;
        };
        this.getDefaultAddressFromWallet = (wallet) => {
            return "0x" + wallet.getAddress().toString("hex");
        };
        this.getDefaultAddressFromMnemonic = (mnemonic) => {
            return this.getDefaultAddressFromWallet(this.getWalletFromMnemonic(mnemonic));
        };
    }
}
//# sourceMappingURL=Toolbox.js.map