"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = y[op[0] & 2 ? "return" : op[0] ? "throw" : "next"]) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [0, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var _this = this;
import * as util from "ethereumjs-util";
export var getAddress = function (web3) {
    return new Promise(function (resolve, reject) {
        web3.eth.getAccounts(function (err, addresses) {
            if (err) {
                reject(err);
            }
            resolve(addresses[0]);
        });
    });
};
export var signMessage = function (web3, address, msgHex) {
    var sigPromise = new Promise(function (resolve, reject) {
        web3.eth.sign(address, msgHex, function (err, signature) {
            if (err) {
                reject(err);
            }
            else {
                resolve(signature);
            }
        });
    });
    return sigPromise
        .then(function (signature) {
        return signature;
    })
        .catch(function () {
        return null;
    });
};
export var recoverAddressFromSig = function (messageBuffer, sig) {
    var _a = util.fromRpcSig(sig), v = _a.v, r = _a.r, s = _a.s;
    var pubKey = util.ecrecover(messageBuffer, v, r, s);
    var addrBuf = util.pubToAddress(pubKey);
    var addr = util.bufferToHex(addrBuf);
    // console.log(address, addr);
    return addr;
};
export var prepareStringForSigning = function (value) {
    var msg = new Buffer(value);
    var msgHex = "0x" + msg.toString("hex");
    return {
        msg: msg,
        msgHex: msgHex
    };
};
export var getSha3BufferOfPrefixedMessage = function (msg) {
    var prefix = new Buffer("\x19Ethereum Signed Message:\n");
    var prefixedMsgBuffer = util.sha3(Buffer.concat([prefix, new Buffer(String(msg.length)), new Buffer(msg)]));
    return prefixedMsgBuffer;
};
export var getSha3BufferOfMessage = function (msg) {
    return util.toBuffer(util.sha3(msg));
};
export var getMessageSignatureWithMeta = function (web3, address, message) { return __awaiter(_this, void 0, void 0, function () {
    var msgObj, sig, prefixedMessageBuffer, messageBuffer, isPrefixed, finalMessageBuffer;
    return __generator(this, function (_a) {
        switch (_a.label) {
            case 0:
                msgObj = prepareStringForSigning(message);
                return [4 /*yield*/, signMessage(web3, address, msgObj.msgHex)];
            case 1:
                sig = _a.sent();
                if (sig === null) {
                    throw new Error("Cannot sign message with address. Consider if this address is a node or wallet address, and how web3 provider engine has been initialized.");
                }
                prefixedMessageBuffer = getSha3BufferOfPrefixedMessage(msgObj.msg);
                messageBuffer = getSha3BufferOfMessage(msgObj.msg);
                if (recoverAddressFromSig(prefixedMessageBuffer, sig) === address) {
                    isPrefixed = true;
                }
                if (recoverAddressFromSig(messageBuffer, sig) === address) {
                    isPrefixed = false;
                }
                finalMessageBuffer = isPrefixed ? prefixedMessageBuffer : messageBuffer;
                return [2 /*return*/, {
                        messageBuffer: finalMessageBuffer,
                        messageBufferHex: util.bufferToHex(finalMessageBuffer),
                        messageHex: msgObj.msgHex,
                        signature: sig,
                        address: address,
                        isPrefixed: isPrefixed
                    }];
        }
    });
}); };
//# sourceMappingURL=Common.js.map