"use strict";
import { EventStore } from "./EventStore/EventStore";
import { ReadModel } from "./EventStore/ReadModel/ReadModel";
import { Factory } from "./EventStore/Factory/Factory";
import { Persistence } from "./EventStore/Persistence/Persistence";
import { PatchLogic } from "./EventStore/ReadModel/PatchLogic/PatchLogic";
import { Permissions } from "./EventStore/Permissions/Permissions";
import { TransmuteIpfs } from "./TransmuteIpfs/TransmuteIpfs";
import { Toolbox } from "./Toolbox/Toolbox";
var Web3 = require("web3");
var contract = require("truffle-contract");
var web3;
var accessControlArtifacts = require("../build/contracts/RBAC");
var eventStoreArtifacts = require("../build/contracts/RBACEventStore");
var eventStoreFactoryArtifacts = require("../build/contracts/RBACEventStoreFactory");
var ProviderEngine = require("web3-provider-engine");
var WalletSubprovider = require("web3-provider-engine/subproviders/wallet.js");
var Web3Subprovider = require("web3-provider-engine/subproviders/web3.js");
var config = {
    providerUrl: "http://localhost:8545",
    aca: accessControlArtifacts,
    esa: eventStoreArtifacts,
    esfa: eventStoreFactoryArtifacts
};
var TransmuteFramework = (function () {
    function TransmuteFramework() {
        var _this = this;
        this.config = config;
        this.engine = new ProviderEngine();
        this.web3 = new Web3(this.engine);
        this.TransmuteIpfs = TransmuteIpfs;
        this.Persistence = Persistence;
        this.initWeb3 = function (providerUrl) {
            _this.engine = new ProviderEngine();
            // Not sure about the security of this... seems dangerous...
            if (_this.config.wallet) {
                _this.engine.addProvider(new WalletSubprovider(_this.config.wallet, {}));
            }
            _this.engine.addProvider(new Web3Subprovider(new Web3.providers.HttpProvider(providerUrl)));
            _this.engine.start(); // Required by the provider engine.
            web3 = new Web3(_this.engine);
            _this.web3 = web3;
        };
        this.init = function (confObj) {
            if (confObj === void 0) { confObj = config; }
            _this.config = confObj;
            _this.initWeb3(_this.config.providerUrl);
            try {
                if (window.web3) {
                    console.info("Transmute Framework has detected MetaMask!");
                    _this.web3 = window.web3;
                }
            }
            catch (e) {
                // probably not in browser... ignore..
            }
            if (_this.web3) {
                _this.AccessControlContract = contract(_this.config.aca);
                _this.AccessControlContract.setProvider(_this.web3.currentProvider);
                _this.EventStoreFactoryContract = contract(_this.config.esfa);
                _this.EventStoreFactoryContract.setProvider(_this.web3.currentProvider);
                _this.EventStoreContract = contract(_this.config.esa);
                _this.EventStoreContract.setProvider(_this.web3.currentProvider);
            }
            else {
                console.warn("web3 is not available, install metamask.");
            }
            _this.initAll();
            return _this;
        };
        this.initAll = function () {
            var ipfsConfig = _this.config.ipfsConfig || {
                host: "localhost",
                port: "5001",
                options: {
                    protocol: "http"
                }
            };
            // This is initialized like so because it can be useful outside framework...
            _this.TransmuteIpfs = TransmuteIpfs.init(ipfsConfig);
            _this.Factory = new Factory(_this);
            _this.EventStore = new EventStore(_this);
            _this.ReadModel = new ReadModel(_this);
            _this.PatchLogic = new PatchLogic(_this);
            _this.Permissions = new Permissions(_this);
            _this.Toolbox = new Toolbox(_this);
        };
        this.getAccounts = function () {
            return new Promise(function (resolve, reject) {
                _this.web3.eth.getAccounts(function (err, addresses) {
                    if (err) {
                        reject(err);
                    }
                    else {
                        // console.log('addresses: ', addresses)
                        resolve(addresses);
                    }
                });
            });
        };
    }
    return TransmuteFramework;
}());
export { TransmuteFramework };
export default new TransmuteFramework();
//# sourceMappingURL=transmute-framework.js.map