import { LocalStorage } from './LocalStore/LocalStorage';
export declare namespace Persistence {
    interface ICacheObject {
        expires: string;
        value: Object;
    }
    interface IPersistenceStore {
        getItem: (key: string) => Promise<Object>;
        setItem: (key: string, value: Object) => Promise<Object>;
    }
    const LocalStore: typeof LocalStorage;
    const FireStore: null;
    /**
       * @param {String} key - a key for a stored object
       * @param {IPersistenceStore} store - the persistence method used for the cache
       * @return {Object} a promise for an empty object, the cache is expired
       */
    const expireItem: (key: string, store?: IPersistenceStore) => Promise<any>;
    /**
       * @param {String} key - a key for a stored object
       * @param {Object} value - a an object to be persisted
       * @param {String} expires - an ISO string for when the cache expires
       * @param {IPersistenceStore} store - the persistence method used for the cache
       * @return {Object} a promise for the object at the given key
       */
    const setItem: (key: string, value: Object, expires?: string, store?: IPersistenceStore) => Promise<any>;
    /**
       * @param {String} key - a key for a stored object
       * @param {IPersistenceStore} store - the persistence method used for the cache
       * @return {Object} a promise for the object at the given key
       */
    const getItem: (key: string, store?: IPersistenceStore) => Promise<any>;
}
